@@ -10,13 +10,12 @@ Todo:
 from typing import Callable, Dict, Optional, Sequence, Union
 
 import numpy as np
-from instancelib.instances.memory import MemoryBucketProvider
-from instancelib.instances.text import MemoryTextInstance
+from genbase import Readable, SeedMixin
+from instancelib.instances.memory import DataPoint, MemoryBucketProvider
 from instancelib.labels.base import LabelProvider
 from instancelib.labels.memory import MemoryLabelProvider
 from instancelib.machinelearning.base import AbstractClassifier
 
-from ..default import Readable
 from .embedding import Embedder, TfidfVectorizer
 from .weights import exponential_kernel
 
@@ -40,18 +39,19 @@ class PrototypeSampler(Readable):
     def embedded(self) -> np.ndarray:
         return np.stack(self.instances.bulk_get_vectors(list(self.instances))[-1])
 
-    def _select_from_provider(self, keys: Sequence[int]) -> Sequence[MemoryTextInstance]:
+    def _select_from_provider(self, keys: Sequence[int]) -> Sequence[DataPoint]:
         """Select instances from provider by keys."""
-        return [self.instances[i] for i in keys]
+        id_map = np.array(self.instances)
+        return [self.instances[id_map[i]] for i in keys]
 
-    def prototypes(self, n: int = 5) -> Sequence[MemoryTextInstance]:
+    def prototypes(self, n: int = 5) -> Sequence[DataPoint]:
         """Select `n` prototypes.
 
         Args:
             n (int, optional): Number of prototypes to select. Defaults to 5.
 
         Returns:
-            Sequence[MemoryTextInstance]: List of prototype instances.
+            Sequence[DataPoint]: List of prototype instances.
         """
         raise NotImplementedError('Implemented in subclasses')
 
@@ -59,7 +59,7 @@ class PrototypeSampler(Readable):
         return self.prototypes(*args, **kwargs)
 
 
-class KMedoids(PrototypeSampler):
+class KMedoids(PrototypeSampler, SeedMixin):
     def __init__(self,
                  instances: MemoryBucketProvider,
                  embedder: Embedder = TfidfVectorizer,
@@ -76,12 +76,12 @@ class KMedoids(PrototypeSampler):
             https://scikit-learn-extra.readthedocs.io/en/stable/generated/sklearn_extra.cluster.KMedoids.html
         """
         super().__init__(instances, embedder)
-        self._seed = seed
+        self._seed = self._original_seed = seed
 
     def prototypes(self,
                    n: int = 5,
                    metric: Union[str, Callable] = 'cosine',
-                   **kwargs) -> Sequence[MemoryTextInstance]:
+                   **kwargs) -> Sequence[DataPoint]:
         """Select `n` prototypes (most representative samples) using `k-Medoids`_.
 
         Args:
@@ -91,7 +91,7 @@ class KMedoids(PrototypeSampler):
             **kwargs: Optional arguments passed to `k-Medoids`_ constructor.
 
         Returns:
-            Sequence[MemoryTextInstance]: List of prototype instances.
+            Sequence[DataPoint]: List of prototype instances.
 
         .. _k-Medoids:
             https://scikit-learn-extra.readthedocs.io/en/stable/generated/sklearn_extra.cluster.KMedoids.html
@@ -99,7 +99,7 @@ class KMedoids(PrototypeSampler):
             https://scikit-learn.org/stable/modules/generated/sklearn.metrics.pairwise_distances.html
         """
         from sklearn_extra.cluster import KMedoids
-        kmedoids = KMedoids(n_clusters=n, metric=metric, random_state=self._seed, **kwargs).fit(self.embedded)
+        kmedoids = KMedoids(n_clusters=n, metric=metric, random_state=self.seed, **kwargs).fit(self.embedded)
         return self._select_from_provider(kmedoids.medoid_indices_)
 
 
@@ -130,7 +130,7 @@ class MMDCritic(PrototypeSampler):
         self.K = self.kernel(self.embedded, 1.0 / self.embedded.shape[1])
         self.colsum = np.sum(self.K, axis=0) / self.embedded.shape[1]
 
-    def prototypes(self, n: int = 5) -> Sequence[MemoryTextInstance]:
+    def prototypes(self, n: int = 5) -> Sequence[DataPoint]:
         """Select `n` prototypes (most representatitve instances), using `MMD-critic implementation`_.
 
         Args:
@@ -140,7 +140,7 @@ class MMDCritic(PrototypeSampler):
             ValueError: Cannot select more instances than the total number of instances.
 
         Returns:
-            Sequence[MemoryTextInstance]: List of prototype instances.
+            Sequence[DataPoint]: List of prototype instances.
 
         .. _MMD-critic implementation:
             https://github.com/maxidl/MMD-critic/blob/main/mmd_critic.py
@@ -150,7 +150,7 @@ class MMDCritic(PrototypeSampler):
 
         K = self.K
         colsum = self.colsum.copy() * 2
-        sample_indices = np.array(list(self.instances))
+        sample_indices = np.arange(0, len(self.instances))
         is_selected = np.zeros_like(sample_indices)
         selected = sample_indices[is_selected > 0]
 
@@ -175,7 +175,7 @@ class MMDCritic(PrototypeSampler):
         self._prototypes = self._select_from_provider(selected_in_order)
         return self._prototypes
 
-    def criticisms(self, n: int = 5, regularizer: Optional[str] = None) -> Sequence[MemoryTextInstance]:
+    def criticisms(self, n: int = 5, regularizer: Optional[str] = None) -> Sequence[DataPoint]:
         """Select `n` criticisms (instances not well represented by prototypes), using `MMD-critic implementation`_. 
 
         Args:
@@ -188,7 +188,7 @@ class MMDCritic(PrototypeSampler):
             ValueError: Unknown regularizer or requested more criticisms than there are samples left.
 
         Returns:
-            Sequence[MemoryTextInstance]: List of criticism instances.
+            Sequence[DataPoint]: List of criticism instances.
 
         .. _MMD-critic implementation:
             https://github.com/maxidl/MMD-critic/blob/main/mmd_critic.py
@@ -203,7 +203,8 @@ class MMDCritic(PrototypeSampler):
             raise ValueError('Cannot select more than instances excluding prototypes ',
                              f'({len(self.instances) - len(self._prototypes)})')
 
-        prototypes = np.array([p.identifier for p in self._prototypes])
+        id_map = {instance: id for id, instance in enumerate(self.instances)}
+        prototypes = np.array([id_map[p.identifier] for p in self._prototypes])
 
         K = self.K
         colsum = self.colsum
@@ -224,7 +225,7 @@ class MMDCritic(PrototypeSampler):
             s1 = np.abs(s1)
 
             if regularizer == 'logdet':
-                diag = np.diagonal(K + 1)[candidate_indices]  # TODO: look at hotfix
+                diag = np.diagonal(K + 1)[candidate_indices]
                 if inverse_of_prev_selected is not None:
                     temp = K[selected, :][:, candidate_indices]
                     temp2 = np.dot(inverse_of_prev_selected, temp) 
@@ -255,7 +256,7 @@ class MMDCritic(PrototypeSampler):
     def __call__(self,
                  n_prototypes: int = 5,
                  n_criticisms: int = 5,
-                 regularizer: Optional[str] = None) -> Dict[str, Sequence[MemoryTextInstance]]:
+                 regularizer: Optional[str] = None) -> Dict[str, Sequence[DataPoint]]:
         """Calculate prototypes and criticisms for the provided instances.
 
         Args:
@@ -265,7 +266,7 @@ class MMDCritic(PrototypeSampler):
                 Defaults to None.
 
         Returns:
-            Dict[str, Sequence[MemoryTextInstance]]: Dictionary containing prototypes and criticisms.
+            Dict[str, Sequence[DataPoint]]: Dictionary containing prototypes and criticisms.
         """
         return {'prototypes': self.prototypes(n=n_prototypes),
                 'criticisms': self.criticisms(n=n_criticisms, regularizer=regularizer)}
@@ -329,27 +330,27 @@ class LabelwisePrototypeSampler(Readable):
                           for label in self.labels.labelset}
         self.samplers = self._samplers
 
-    def prototypes(self, n: int = 5) -> Dict[str, Sequence[MemoryTextInstance]]:
+    def prototypes(self, n: int = 5) -> Dict[str, Sequence[DataPoint]]:
         """Select `n` prototypes (most representatitve instances).
 
         Args:
             n (int, optional): Number of prototypes to select. Defaults to 5.
 
         Returns:
-            Dict[str, Sequence[MemoryTextInstance]]: Dictionary with labels and corresponding list of prototypes.
+            Dict[str, Sequence[DataPoint]]: Dictionary with labels and corresponding list of prototypes.
         """
         return {label: sampler.prototypes(n=n)
                 for label, sampler in self._samplers.items()}
 
     def __call__(self,
-                 n: int = 5) -> Dict[str, Dict[str, Sequence[MemoryTextInstance]]]:
+                 n: int = 5) -> Dict[str, Dict[str, Sequence[DataPoint]]]:
         """Generate prototypes for each label.
 
         Args:
             n (int, optional): Number of prototypes to select. Defaults to 5.
 
         Returns:
-            Dict[str, Dict[str, Sequence[MemoryTextInstance]]]: Dictionary with labels and corresponding dictionary 
+            Dict[str, Dict[str, Sequence[DataPoint]]]: Dictionary with labels and corresponding dictionary 
                 containing prototypes.
         """
         return {label: {'prototypes': sampler.prototypes(n=n)}
@@ -407,7 +408,7 @@ class LabelwiseMMDCritic(LabelwisePrototypeSampler):
                          embedder=embedder,
                          kernel=kernel)
 
-    def criticisms(self, n: int = 5, regularizer: Optional[str] = None) -> Dict[str, Sequence[MemoryTextInstance]]:
+    def criticisms(self, n: int = 5, regularizer: Optional[str] = None) -> Dict[str, Sequence[DataPoint]]:
         """Select `n` criticisms (instances not well represented by prototypes).
 
         Args:
@@ -419,7 +420,7 @@ class LabelwiseMMDCritic(LabelwisePrototypeSampler):
             Exception: `MMDCritic.prototypes()` must first be run before being able to determine the criticisms.
 
         Returns:
-            Dict[str, Sequence[MemoryTextInstance]]: Dictionary with labels and corresponding list of criticisms.
+            Dict[str, Sequence[DataPoint]]: Dictionary with labels and corresponding list of criticisms.
         """
         return {label: sampler.criticisms(n=n, regularizer=regularizer)
                 for label, sampler in self._samplers.items()}
@@ -427,7 +428,7 @@ class LabelwiseMMDCritic(LabelwisePrototypeSampler):
     def __call__(self,
                  n_prototypes: int = 5,
                  n_criticisms: int = 5,
-                 regularizer: Optional[str] = None) -> Dict[str, Dict[str, Sequence[MemoryTextInstance]]]:
+                 regularizer: Optional[str] = None) -> Dict[str, Dict[str, Sequence[DataPoint]]]:
         """Generate prototypes and criticisms for each label.
 
         Args:
@@ -437,7 +438,7 @@ class LabelwiseMMDCritic(LabelwisePrototypeSampler):
                 Defaults to None.
 
         Returns:
-            Dict[str, Dict[str, Sequence[MemoryTextInstance]]]: Dictionary with labels and corresponding dictionary 
+            Dict[str, Dict[str, Sequence[DataPoint]]]: Dictionary with labels and corresponding dictionary 
                 containing prototypes and criticisms.
         """
         return {label: sampler(n_prototypes=n_prototypes, n_criticisms=n_criticisms, regularizer=regularizer)
